
'*******************************************************************
' Stack Fits Images.txt
' James L. Jones, 2005                                                               
'                                                                                          
'This script utilizes MaxIm to stack images in the Fits Format.          
'The images to be stacked are placed in a folder.  Nothing else        
'Can be in the folder.  Drag and drop the image folder to the           
'script icon.  The script will tell you how many images you have      
'in the folder and ask how many images you want in each stack.
'Sit back and watch.                                                                                                             
'The stacked images and a log file will be placed in the image folder.
' 
'Thanks to Bob Denny for showing me the Drag and Drop thing.
'                                                                 
'Modified 2019                       
'******************************************************************

Option Explicit
Dim Doc, Apps, AllDocs, Path, collArgs, fso, f, fc, count, fl, SavePath
Dim Images(1500), Img(1200), ImgDup(1200), i, ii, j, Item(10), N, jint, jmod, k, StackIndex
Dim LineTxt(1500), ImageText(1500), DateTime, MidPoint, MidPointCor, ts, cso, TxtLog, indx, jj, kk
Dim Name, TimeShift, DateTimeShifted, DateTimeCor, ExpTime, ISOSPEED, CCDTemper, T

Set collArgs = WScript.Arguments
Call Drag_Drop()

'*********************************************
'     Get Collection of file Names                        
'*********************************************
Set fso = CreateObject("Scripting.FileSystemObject")
Set f = fso.GetFolder(collArgs(0))
Set fc = f.Files
Call MoveColl()
Call InputData()
Call Calculate()

'**********************************************************
'    jint = number of whole stacks                       
'    N = Number of images in each whole stack            
'    jmod = number of images in partial stack, if any    
'    k = first image in stack                            
'    k+N-1 = Last image in stack                         
'    ii = MaxIm Image Number                             
'    i = Image being oppened from Images()               
'**********************************************************
Indx = 0
k = 0
Call CloseAllImages()

If Not fso.FolderExists(collArgs(0) & "\G2G3\") Then
	fso.CreateFolder(collArgs(0) & "\G2G3\")
End If

For StackIndex = 0 To jint-1
	ii = 0
	
		For i = k To k+N-1
			Call  OpenImage(i, ii)
			ii = ii + 1
		Next
	k = k + N
	Call Stack_Save()
	Call CloseAllImages()
Next
Call CreateLog()
Msgbox "Finished!", , "Finished"


'*********************************************
'                Drag and Drop              
'*********************************************
Sub Drag_Drop()
count = 0
If collArgs.Count = 0 Then
      WScript.Echo _
"Drag a folder of FITS images to this icon to stack."
    	WScript.Quit
End If
End Sub

'*****************************************************************
'    Move collection of file names (fc) to array (Images())     
'*****************************************************************
Sub MoveColl()
count = fc.Count 
i = 0
For Each fl In fc
	Images(i) = fl.name
	i=i+1
Next
End Sub

'***********************************************
'    Input number of images in each stack      
'***********************************************
Sub InputData()
N = 1
'TimeShift = cdate(InputBox("Put difference in hours between Time in Exif and UT (your Timezone and daylightsaving):","Enter Value", 0)/24)

    TimeShift = InputBox("What difference in hours is between Time in Exif and UT" _
	& " (your Timezone + daylight saving)?", "Input Data", 0)
    Call InputDataCancel(TimeShift)
    TimeShift = cdate(TimeShift/24)
End Sub

'******************************************************************************
'				Input Data Cancel
'******************************************************************************
Sub InputDataCancel(value)
    If IsEmpty(value) Then
        MsgBox "Operations have been cancelled"
        WScript.Quit
    End If
End Sub

'******************************************
'          Subroutine CloseAllImages               
'******************************************
Sub CloseAllImages()
Set Apps = CreateObject("MaxIm.Application")
Apps.CloseAll()
End Sub

'***********************************************
'           Subroutine Calculate              
' Calculate number of stacks (jint) and the    
' number of images In the partial stack       
'***********************************************
Sub Calculate()
	jint = count \ N		'jint = number of whole stacks
	'jmod = count Mod N		'jmod = number of images in partial stack, if any
End Sub

'********************************
'    Subroutine Stack_Save     
'********************************
Sub Stack_save()
SavePath = collArgs(0) & "\G2G3\" & Name
Set Img(ii) = CreateObject("MaxIm.Document")

	'Document.CombineImages ( Mode, Bicubic, Output, Normalize[, Images ]) :
		'AlignType Mode: 0 - autocorrelation, 1 - automatic matching of star brightnesses and separations, 2 - perform no alignment, 3 - WCS information in the FITS header, 5 - aligns centroids of the brightest star found in each image
		'CombineType Output: 0 - sum, 1 - average, 2 - median, 3 - sigma-clip algorithm, 4 - standard-deviation masking
		'Boolean Normalize - prenormalizes images so that medians are comparable (ignored if output is calculated by sum or average methods)
		'Collection Images - [optional] a Collection or Dictionary object whose members are the images to be aligned
Img(ii).CombineImages 2, False, 1, False

Call ReadKeyWords2()  
ImageText(Indx) = Name & "    DATE-OBS: " & DateTime & "    MIDPOINT: " & MidPoint & "     ExpTime: " & ExpTime & "sec" & "    " & ISOSPEED & "    CCD-TEMP: " & CCDTemper  

	'Document.SaveFile ( FilePath, FileFormat, AutoStretch[, SizeFormat, CompressionType ] :
		'FileFormat: 3 -  FITS, 4 - simple Raw data;
		'SizeFormat: 0 - 8 bits,  1 - 16 bits, 2 - 32 bits, 3 - 4bytes in IEEE floating point format
Img(ii).SaveFile SavePath, 3, False, 3, False  

Img(ii).Close
Set Img(ii) = Nothing
Indx = Indx + 1		
End Sub


'********************************
'     Subroutine OpenImage()   
'********************************
Sub OpenImage(i, ii)
Set Img(ii) = CreateObject("MaxIm.Document")
Path = collArgs(0) & "\" & Images(i)
Name = Replace(Images(i), ".fit", "_G2G3.fit")
Img(ii).OpenFile (Path)
'Img(ii).Calibrate				'CALIBRATION
Set ImgDup(ii) = Img(ii).Duplicate
Img(ii).ExtractBayerPlane 2
ImgDup(ii).ExtractBayerPlane 3
Call ReadKeyWords()
	'WScript.Echo i, ii, StackIndex, Images(i)
ImageText(Indx) = Images(i) &  "     " & DateTime & "     ExpTime: " & ExpTime & "sec" & "    CCD-TEMP: " & CCDTemper
	'WScript.Echo Indx, ImageText(Indx)
Indx = Indx + 1
Set Img(ii) = Nothing
Set ImgDup(ii) = Nothing
End Sub

'*********************************
'   Subroutine ReadKeyWords()   
'*********************************
Sub ReadKeyWords()
DateTime = (Img(ii).GetFITSKey("DATE-OBS"))  
ExpTime = (Img(ii).GetFITSKey("EXPTIME"))  
CCDTemper = (Img(ii).GetFITSKey("CCD-TEMP"))
End Sub

'*********************************
'   Subroutine ReadKeyWords2()   
'*********************************
Sub ReadKeyWords2()  

Call ReadKeyWords()  

MidPoint = (Img(ii).GetFITSKey("MIDPOINT"))  
ISOSPEED = (Img(ii).GetFITSKey("ISOSPEED"))  

Call MidPointCorrector(MidPoint)  
Img(ii).SetFITSKey "MIDPOINT",MidPointCor  
MidPoint = (Img(ii).GetFITSKey("MIDPOINT"))

If TimeShift <> 0 Then
	Call DateTimeShifter(DateTime)  
	Img(ii).SetFITSKey "DATE-OBS",DateTimeShifted
	DateTime = (Img(ii).GetFITSKey("DATE-OBS"))
End If

End Sub

'*********************************
'   Subroutine MidPointCorrector()   
'*********************************
Sub MidPointCorrector(MidPoint)
T = cdate(Replace(MidPoint,"T"," ")) + cdate(ExpTime / 2 / 86400) - TimeShift  
MidPointCor = DateToDateTimeConverter(T)
End Sub

'*********************************
'   Subroutine DateTimeShifter()   
'*********************************
Sub DateTimeShifter(DateTime)
T = cdate(Replace(DateTime,"T"," ")) - TimeShift  
DateTimeShifted = DateToDateTimeConverter(T)
End Sub

'****************************************
'   Subroutine DateToDateTimeConverter()       
'****************************************
Function DateToDateTimeConverter(T)
T = Year(T) & "-" & Pd(Month(T),2) & "-" & Pd(Day(T),2) & "T" & Pd(DatePart("h",T),2) & ":" & Pd(DatePart("n",T),2) & ":" & Pd(DatePart("s",T),2)
DateToDateTimeConverter = T
End Function

'*********************************
'   Subroutine CreateLog()       
'*********************************
Sub CreateLog()
Set cso = CreateObject("Scripting.FileSystemObject")
TxtLog = collArgs(0) & "\_G2G3_log.txt"
Set ts = cso.OpenTextFile(TxtLog, 2, True)
For kk = 0 To Indx
	ts.WriteLine(ImageText(kk))
Next		
ts.close
End Sub

'******************************************
'       Subroutine pd(n, totalDigits)
' Help to generate your custom date format
'******************************************
Function pd(n, totalDigits) 
    if totalDigits > len(n) then 
        pd = String(totalDigits-len(n),"0") & n 
    else 
        pd = n 
    end if 
End Function 

